@@ -1588,7 +1588,8 @@ def module(out, allInOne, classes, imports, cppdir, moduleName,
     line(out, 0, '}')
 
 
-def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
+def compile(env, jccPath, output, moduleName,
+            generate, install, dist, debug, jars,
             version, prefix, root, install_dir, home_dir, use_distutils,
             shared, compiler, modules, wheel, wininst, find_jvm_dll, arch,
             generics, resources, imports, use_full_names, egg_info,
@@ -1607,25 +1608,26 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
         with_setuptools = False
 
     extname = '_%s' %(moduleName)
-
+    if not os.path.isdir(os.path.join(output, extname)):
+        raise FileNotFoundError(
+            "%s directory not found, module %s needs to be compiled first" %(
+                os.path.join(output, extname), extname))
+    
     modulePath = os.path.join(output, moduleName)
     if not os.path.isdir(modulePath):
         os.makedirs(modulePath)
 
     out = open(os.path.join(modulePath, '__init__.py'), 'w')
     line(out)
-    if shared:
-        line(out, 0, "import os, sys")
-        line(out)
+    line(out, 0, "import os")
+    if find_jvm_dll:
+        line(out, 0, "import sys")
         line(out, 0, "if sys.platform == 'win32':")
-        if find_jvm_dll:
-            line(out, 1, "from jcc.windows import add_jvm_dll_directory_to_path")
-            line(out, 1, "add_jvm_dll_directory_to_path()")
-        line(out, 1, "import jcc")
-        line(out, 0, "from . import %s", extname)
-    else:
-        line(out, 0, "import os")
-        line(out, 0, "from . import %s", extname)
+        line(out, 1, "from .windows import add_jvm_dll_directory_to_path")
+        line(out, 1, "add_jvm_dll_directory_to_path('%s')" %(find_jvm_dll))
+        modules.append(os.path.join(
+            os.path.dirname(os.path.realpath(__file__)), 'windows.py'))
+    line(out, 0, "from . import %s", extname)
     line(out)
     line(out, 0, '__module_dir__ = os.path.abspath(os.path.dirname(__file__))')
 
@@ -1870,23 +1872,40 @@ def compile(env, jccPath, output, moduleName, install, dist, debug, jars,
                 i += 1
         config_vars['CFLAGS'] = ' '.join(cflags)
 
-    print("extension args = %s" % args)
-    extensions = [Extension('.'.join([moduleName, extname]), **args)]
-    script_args.extend(extra_setup_args)
+    if generate:
+        out = open(os.path.join(".", "setup.py"), "w")
+        line(out, 0, 'from setuptools import setup, Extension')
+        line(out, 0, 'setup(')
+        line(out, 1, 'name="%s",', moduleName)
+        line(out, 1, 'packages=%s,', packages)
+        line(out, 1, 'package_dir={"%s": "%s"},', moduleName, modulePath)
+        line(out, 1, 'package_data={"%s": %s},', moduleName, package_data)
+        line(out, 1, 'ext_modules=[Extension("%s.%s",', moduleName, extname)
+        for arg, value in args.items():
+            line(out, 2, '%s=%s,', arg, value)
+        line(out, 1, ")],")
+        if version:
+            line(out, 1, 'version="%s",', version)
+        line(out, 1, 'zip_safe=False,')
+        line(out, 0, ')')
+        out.close()
+    else:
+        print("extension args = %s" % args)
+        extensions = [Extension('.'.join([moduleName, extname]), **args)]
+        script_args.extend(extra_setup_args)
 
-    args = {
-        'name': moduleName,
-        'packages': packages,
-        'package_dir': {moduleName: modulePath},
-        'package_data': {moduleName: package_data},
-        'ext_modules': extensions,
-        'script_args': script_args
-    }
-    if version:
-        args['version'] = version
-    if with_setuptools:
-        args['zip_safe'] = False
+        setup_args = {
+            'name': moduleName,
+            'packages': packages,
+            'package_dir': {moduleName: modulePath},
+            'package_data': {moduleName: package_data},
+            'ext_modules': extensions,
+            'script_args': script_args
+        }
+        if version:
+            setup_args['version'] = version
+        if with_setuptools:
+            setup_args['zip_safe'] = False
 
-    print("setup args = %s" % args)
-
-    setup(**args)
+            print("setup args = %s" % setup_args)
+            setup(**setup_args)
